import {ApplicationState, ContentNegotiation, Resource} from '../common';
import {DataSource} from './data-source';
import {BaseSchema} from 'valibot';
import {Middleware} from './http/server';

export interface BackendState {
	app: ApplicationState;
	dataSource: DataSource;
	cn: ContentNegotiation;
	showTotalItemCountOnGetCollection: boolean;
	throwsErrorOnDeletingNotFound: boolean;
	checksSerializersOnDelete: boolean;
	showTotalItemCountOnCreateItem: boolean;
}

export interface RequestContext {}

export type RequestDecorator = (req: RequestContext) => RequestContext | Promise<RequestContext>;

export type ParamRequestDecorator<Params extends Array<unknown> = []> = (...args: Params) => RequestDecorator;

export type Method = 'GET' | 'POST' | 'PUT' | 'PATCH' | 'DELETE' | 'OPTIONS' | 'HEAD';

export interface AllowedMiddlewareSpecification<Schema extends BaseSchema = BaseSchema> {
	method: Method;
	middleware: Middleware;
	constructBodySchema?: (resource: Resource<Schema>, resourceId?: string) => BaseSchema;
	allowed: (resource: Resource<Schema>) => boolean;
}
