import {HttpMiddlewareError, Middleware, PlainResponse} from '../server';
import {LinkMap} from '../utils';
import {constants} from 'http2';
import {AllowedMiddlewareSpecification} from '../../common';

export const handleGetRoot: Middleware = (req) => {
	const { backend, basePath } = req;

	const data = {
		name: backend.app.name
	};

	const registeredResources = Array.from(backend.app.resources);
	const availableResources = registeredResources.filter((r) => (
		r.state.canFetchCollection
		|| r.state.canCreate
	));

	const headers: Record<string, string> = {};
	if (availableResources.length > 0) {
		// https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Link
		headers['Link'] = new LinkMap(
			availableResources.map((r) => ({
				url: `${basePath}/${r.state.routeName}`,
				params: {
					rel: 'related',
					name: r.state.routeName,
				},
			}))
		)
			.toString();
	}

	return new PlainResponse({
		headers,
		statusMessage: 'ok',
		statusCode: constants.HTTP_STATUS_OK,
		body: data
	});
};

export const handleOptions = (middlewares: AllowedMiddlewareSpecification[]): Middleware => () => {
	if (middlewares.length > 0) {
		return new PlainResponse({
			headers: {
				'Allow': middlewares.flatMap((m) => m.method === 'GET' ? [m.method, 'HEAD'] : [m.method]).join(', '),
			},
			statusMessage: 'ok',
			statusCode: constants.HTTP_STATUS_NO_CONTENT,
		});
	}

	// TODO add option for custom error handler
	throw new HttpMiddlewareError('methodNotAllowed', {
		statusCode: constants.HTTP_STATUS_METHOD_NOT_ALLOWED,
	});
};
